/**
 * @file
 * packery.settings.js
 */

(function($) {

  Drupal.behaviors.packery = {
    attach: function(context, settings) {
      for (var selector in settings.packery) {
        this.attachInit(selector, settings.packery[selector]);
      }
    },

    /**
     * Packery selector and settings to invoke.
     *
     * @param selector htmlSelector
     * @param options object
     */
    attachInit: function(selector, options) {
      var element = $('#' + selector);
      this.attachPackery(element, options.settings);
    },

    /**
     * Attach and initiate Packery on element.
     *
     * @param element htmlElement
     * @param settings object
     */
    attachPackery: function(element, settings) {
      // Provide settings alter before initialisation.
      $(document).trigger('packery.alterSettings', settings);

      var container = element.packery(settings);
      this.attachImagesLoaded(container, settings.imagesloaded);

      // Provide access to packery instance.
      $(document).trigger('packery.alterInstance', [container]);
    },

    /**
     * Provide support for imagesLoaded plugin.
     *
     * @param packery object
     * @param imagesLoaded boolean
     */
    attachImagesLoaded: function(container, imagesLoaded) {
      if (imagesLoaded) {
        container.imagesLoaded(function() {
          container.packery();
        });
      }
    }

  };

}(jQuery));
